# -*- coding: utf-8 -*-
import sys


_ver = sys.version_info
is_py3 = _ver[0] == 3
is_py2 = _ver[0] == 2


if is_py2:
    bytes = lambda x: chr(x[0])  # @ReservedAssignment
elif is_py3:
    bytes = bytes  # @ReservedAssignment



# -*- coding: utf-8 -*-
__all__ = ['Error', 'MalformedMKVError', 'ParserError', 'ReadError', 'SizeError']


class Error(Exception):
    """Base class for enzyme exceptions"""
    pass


class MalformedMKVError(Error):
    """Wrong or malformed element found"""
    pass


class ParserError(Error):
    """Base class for exceptions in parsers"""
    pass


class ReadError(ParserError):
    """Unable to correctly read"""
    pass


class SizeError(ParserError):
    """Mismatch between the type of the element and the size of its data"""
    pass



# -*- coding: utf-8 -*-
from enzyme.parsers import ebml
import io
import os.path
import requests
import unittest
import yaml
import zipfile


# Test directory
TEST_DIR = os.path.join(os.path.dirname(__file__), os.path.splitext(__file__)[0])

# EBML validation directory
EBML_VALIDATION_DIR = os.path.join(os.path.dirname(__file__), 'parsers', 'ebml')


def setUpModule():
    if not os.path.exists(TEST_DIR):
        r = requests.get('http://downloads.sourceforge.net/project/matroska/test_files/matroska_test_w1_1.zip')
        with zipfile.ZipFile(io.BytesIO(r.content), 'r') as f:
            f.extractall(TEST_DIR)


class EBMLTestCase(unittest.TestCase):
    def setUp(self):
        self.stream = io.open(os.path.join(TEST_DIR, 'test1.mkv'), 'rb')
        with io.open(os.path.join(EBML_VALIDATION_DIR, 'test1.mkv.yml'), 'r') as yml:
            self.validation = yaml.safe_load(yml)
        self.specs = ebml.get_matroska_specs()

    def tearDown(self):
        self.stream.close()

    def check_element(self, element_id, element_type, element_name, element_level, element_position, element_size, element_data, element,
                      ignore_element_types=None, ignore_element_names=None, max_level=None):
        """Recursively check an element"""
        # base
        self.assertTrue(element.id == element_id)
        self.assertTrue(element.type == element_type)
        self.assertTrue(element.name == element_name)
        self.assertTrue(element.level == element_level)
        self.assertTrue(element.position == element_position)
        self.assertTrue(element.size == element_size)
        # Element
        if not isinstance(element_data, list):
            self.assertTrue(type(element) == ebml.Element)
            if element_type != ebml.BINARY:
                self.assertTrue(element.data == element_data)
            return
        # MasterElement
        if ignore_element_types is not None:  # filter validation on element types
            element_data = [e for e in element_data if e[1] not in ignore_element_types]
        if ignore_element_names is not None:  # filter validation on element names
            element_data = [e for e in element_data if e[2] not in ignore_element_names]
        if element.level == max_level:  # special check when maximum level is reached
            self.assertTrue(element.data is None)
            return
        self.assertTrue(len(element.data) == len(element_data))
        for i in range(len(element.data)):
            self.check_element(element_data[i][0], element_data[i][1], element_data[i][2], element_data[i][3],
                               element_data[i][4], element_data[i][5], element_data[i][6], element.data[i], ignore_element_types,
                               ignore_element_names, max_level)

    def test_parse_full(self):
        result = ebml.parse(self.stream, self.specs)
        self.assertTrue(len(result) == len(self.validation))
        for i in range(len(self.validation)):
            self.check_element(self.validation[i][0], self.validation[i][1], self.validation[i][2], self.validation[i][3],
                               self.validation[i][4], self.validation[i][5], self.validation[i][6], result[i])

    def test_parse_ignore_element_types(self):
        ignore_element_types = [ebml.INTEGER, ebml.BINARY]
        result = ebml.parse(self.stream, self.specs, ignore_element_types=ignore_element_types)
        self.validation = [e for e in self.validation if e[1] not in ignore_element_types]
        self.assertTrue(len(result) == len(self.validation))
        for i in range(len(self.validation)):
            self.check_element(self.validation[i][0], self.validation[i][1], self.validation[i][2], self.validation[i][3],
                               self.validation[i][4], self.validation[i][5], self.validation[i][6], result[i], ignore_element_types=ignore_element_types)

    def test_parse_ignore_element_names(self):
        ignore_element_names = ['EBML', 'SimpleBlock']
        result = ebml.parse(self.stream, self.specs, ignore_element_names=ignore_element_names)
        self.validation = [e for e in self.validation if e[2] not in ignore_element_names]
        self.assertTrue(len(result) == len(self.validation))
        for i in range(len(self.validation)):
            self.check_element(self.validation[i][0], self.validation[i][1], self.validation[i][2], self.validation[i][3],
                               self.validation[i][4], self.validation[i][5], self.validation[i][6], result[i], ignore_element_names=ignore_element_names)

    def test_parse_max_level(self):
        max_level = 3
        result = ebml.parse(self.stream, self.specs, max_level=max_level)
        self.validation = [e for e in self.validation if e[3] <= max_level]
        self.assertTrue(len(result) == len(self.validation))
        for i in range(len(self.validation)):
            self.check_element(self.validation[i][0], self.validation[i][1], self.validation[i][2], self.validation[i][3],
                               self.validation[i][4], self.validation[i][5], self.validation[i][6], result[i], max_level=max_level)


def generate_yml(filename, specs):
    """Generate  a validation file for the test video"""
    def _to_builtin(elements):
        """Recursively convert elements to built-in types"""
        result = []
        for e in elements:
            if isinstance(e, ebml.MasterElement):
                result.append((e.id, e.type, e.name, e.level, e.position, e.size, _to_builtin(e.data)))
            else:
                result.append((e.id, e.type, e.name, e.level, e.position, e.size, None if isinstance(e.data, io.BytesIO) else e.data))
        return result
    video = io.open(os.path.join(TEST_DIR, filename), 'rb')
    yml = io.open(os.path.join(EBML_VALIDATION_DIR, filename + '.yml'), 'w')
    yaml.safe_dump(_to_builtin(ebml.parse(video, specs)), yml)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(EBMLTestCase))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())



# -*- coding: utf-8 -*-
__title__ = 'enzyme'
__version__ = '0.4.0'
__author__ = 'Antoine Bertin'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2013 Antoine Bertin'

import logging
from .exceptions import *
from .mkv import *


logging.getLogger(__name__).addHandler(logging.NullHandler())



# -*- coding: utf-8 -*-
from . import test_mkv, test_parsers
import unittest


suite = unittest.TestSuite([test_mkv.suite(), test_parsers.suite()])


if __name__ == '__main__':
    unittest.TextTestRunner().run(suite)



# -*- coding: utf-8 -*-
from ...exceptions import ReadError
from .readers import *
from pkg_resources import resource_stream  # @UnresolvedImport
from xml.dom import minidom
import logging


__all__ = ['INTEGER', 'UINTEGER', 'FLOAT', 'STRING', 'UNICODE', 'DATE', 'MASTER', 'BINARY',
           'SPEC_TYPES', 'READERS', 'Element', 'MasterElement', 'parse', 'parse_element',
           'get_matroska_specs']
logger = logging.getLogger(__name__)


# EBML types
INTEGER, UINTEGER, FLOAT, STRING, UNICODE, DATE, MASTER, BINARY = range(8)

# Spec types to EBML types mapping
SPEC_TYPES = {
    'integer': INTEGER,
    'uinteger': UINTEGER,
    'float': FLOAT,
    'string': STRING,
    'utf-8': UNICODE,
    'date': DATE,
    'master': MASTER,
    'binary': BINARY
}

# Readers to use per EBML type
READERS = {
    INTEGER: read_element_integer,
    UINTEGER: read_element_uinteger,
    FLOAT: read_element_float,
    STRING: read_element_string,
    UNICODE: read_element_unicode,
    DATE: read_element_date,
    BINARY: read_element_binary
}


class Element(object):
    """Base object of EBML

    :param int id: id of the element, best represented as hexadecimal (0x18538067 for Matroska Segment element)
    :param type: type of the element
    :type type: :data:`INTEGER`, :data:`UINTEGER`, :data:`FLOAT`, :data:`STRING`, :data:`UNICODE`, :data:`DATE`, :data:`MASTER` or :data:`BINARY`
    :param string name: name of the element
    :param int level: level of the element
    :param int position: position of element's data
    :param int size: size of element's data
    :param data: data as read by the corresponding :data:`READERS`

    """
    def __init__(self, id=None, type=None, name=None, level=None, position=None, size=None, data=None):
        self.id = id
        self.type = type
        self.name = name
        self.level = level
        self.position = position
        self.size = size
        self.data = data

    def __repr__(self):
        return '<%s [%s, %r]>' % (self.__class__.__name__, self.name, self.data)


class MasterElement(Element):
    """Element of type :data:`MASTER` that has a list of :class:`Element` as its data

    :param int id: id of the element, best represented as hexadecimal (0x18538067 for Matroska Segment element)
    :param string name: name of the element
    :param int level: level of the element
    :param int position: position of element's data
    :param int size: size of element's data
    :param data: child elements
    :type data: list of :class:`Element`

    :class:`MasterElement` implements some magic methods to ease manipulation. Thus, a MasterElement supports
    the `in` keyword to test for the presence of a child element by its name and gives access to it
    with a container getter::

        >>> ebml_element = parse(open('test1.mkv', 'rb'), get_matroska_specs())[0]
        >>> 'EBMLVersion' in ebml_element
        False
        >>> 'DocType' in ebml_element
        True
        >>> ebml_element['DocType']
        Element(DocType, u'matroska')

    """
    def __init__(self, id=None, name=None, level=None, position=None, size=None, data=None):
        super(MasterElement, self).__init__(id, MASTER, name, level, position, size, data)

    def load(self, stream, specs, ignore_element_types=None, ignore_element_names=None, max_level=None):
        """Load children :class:`Elements <Element>` with level lower or equal to the `max_level`
        from the `stream` according to the `specs`

        :param stream: file-like object from which to read
        :param dict specs: see :ref:`specs`
        :param int max_level: maximum level for children elements
        :param list ignore_element_types: list of element types to ignore
        :param list ignore_element_names: list of element names to ignore
        :param int max_level: maximum level of elements

        """
        self.data = parse(stream, specs, self.size, ignore_element_types, ignore_element_names, max_level)

    def get(self, name, default=None):
        """Convenience method for ``master_element[name].data if name in master_element else default``

        :param string name: the name of the child to get
        :param default: default value if `name` is not in the :class:`MasterElement`
        :return: the data of the child :class:`Element` or `default`

        """
        if name not in self:
            return default
        element = self[name]
        if element.type == MASTER:
            raise ValueError('%s is a MasterElement' % name)
        return element.data

    def __getitem__(self, key):
        if isinstance(key, int):
            return self.data[key]
        children = [e for e in self.data if e.name == key]
        if not children:
            raise KeyError(key)
        if len(children) > 1:
            raise KeyError('More than 1 child with key %s (%d)' % (key, len(children)))
        return children[0]

    def __contains__(self, item):
        return len([e for e in self.data if e.name == item]) > 0

    def __iter__(self):
        return iter(self.data)


def parse(stream, specs, size=None, ignore_element_types=None, ignore_element_names=None, max_level=None):
    """Parse a stream for `size` bytes according to the `specs`

    :param stream: file-like object from which to read
    :param size: maximum number of bytes to read, None to read all the stream
    :type size: int or None
    :param dict specs: see :ref:`specs`
    :param list ignore_element_types: list of element types to ignore
    :param list ignore_element_names: list of element names to ignore
    :param int max_level: maximum level of elements
    :return: parsed data as a tree of :class:`~enzyme.parsers.ebml.core.Element`
    :rtype: list

    .. note::
        If `size` is reached in a middle of an element, reading will continue
        until the element is fully parsed.

    """
    ignore_element_types = ignore_element_types if ignore_element_types is not None else []
    ignore_element_names = ignore_element_names if ignore_element_names is not None else []
    start = stream.tell()
    elements = []
    while size is None or stream.tell() - start < size:
        try:
            element = parse_element(stream, specs)
            if element is None:
                continue
            logger.debug('%s %s parsed', element.__class__.__name__, element.name)
            if element.type in ignore_element_types or element.name in ignore_element_names:
                logger.info('%s %s ignored', element.__class__.__name__, element.name)
                if element.type == MASTER:
                    stream.seek(element.size, 1)
                continue
            if element.type == MASTER:
                if max_level is not None and element.level >= max_level:
                    logger.info('Maximum level %d reached for children of %s %s', max_level, element.__class__.__name__, element.name)
                    stream.seek(element.size, 1)
                else:
                    logger.debug('Loading child elements for %s %s with size %d', element.__class__.__name__, element.name, element.size)
                    element.data = parse(stream, specs, element.size, ignore_element_types, ignore_element_names, max_level)
            elements.append(element)
        except ReadError:
            if size is not None:
                raise
            break
    return elements


def parse_element(stream, specs, load_children=False, ignore_element_types=None, ignore_element_names=None, max_level=None):
    """Extract a single :class:`Element` from the `stream` according to the `specs`

    :param stream: file-like object from which to read
    :param dict specs: see :ref:`specs`
    :param bool load_children: load children elements if the parsed element is a :class:`MasterElement`
    :param list ignore_element_types: list of element types to ignore
    :param list ignore_element_names: list of element names to ignore
    :param int max_level: maximum level for children elements
    :return: the parsed element
    :rtype: :class:`Element`

    """
    ignore_element_types = ignore_element_types if ignore_element_types is not None else []
    ignore_element_names = ignore_element_names if ignore_element_names is not None else []
    element_id = read_element_id(stream)
    if element_id is None:
        raise ReadError('Cannot read element id')
    element_size = read_element_size(stream)
    if element_size is None:
        raise ReadError('Cannot read element size')
    if element_id not in specs:
        logger.error('Element with id 0x%x is not in the specs' % element_id)
        stream.seek(element_size, 1)
        return None
    element_type, element_name, element_level = specs[element_id]
    if element_type == MASTER:
        element = MasterElement(element_id, element_name, element_level, stream.tell(), element_size)
        if load_children:
            element.data = parse(stream, specs, element.size, ignore_element_types, ignore_element_names, max_level)
    else:
        element = Element(element_id, element_type, element_name, element_level, stream.tell(), element_size)
        element.data = READERS[element_type](stream, element_size)
    return element


def get_matroska_specs(webm_only=False):
    """Get the Matroska specs

    :param bool webm_only: load *only* WebM specs
    :return: the specs in the appropriate format. See :ref:`specs`
    :rtype: dict

    """
    specs = {}
    with resource_stream(__name__, 'specs/matroska.xml') as resource:
        xmldoc = minidom.parse(resource)
        for element in xmldoc.getElementsByTagName('element'):
            if not webm_only or element.hasAttribute('webm') and element.getAttribute('webm') == '1':
                specs[int(element.getAttribute('id'), 16)] = (SPEC_TYPES[element.getAttribute('type')], element.getAttribute('name'), int(element.getAttribute('level')))
    return specs



# -*- coding: utf-8 -*-
from ...compat import bytes
from ...exceptions import ReadError, SizeError
from datetime import datetime, timedelta
from io import BytesIO
from struct import unpack


__all__ = ['read_element_id', 'read_element_size', 'read_element_integer', 'read_element_uinteger',
           'read_element_float', 'read_element_string', 'read_element_unicode', 'read_element_date',
           'read_element_binary']


def _read(stream, size):
    """Read the `stream` for *exactly* `size` bytes and raise an exception if
    less than `size` bytes are actually read

    :param stream: file-like object from which to read
    :param int size: number of bytes to read
    :raise ReadError: when less than `size` bytes are actually read
    :return: read data from the `stream`
    :rtype: bytes

    """
    data = stream.read(size)
    if len(data) < size:
        raise ReadError('Less than %d bytes read (%d)' % (size, len(data)))
    return data


def read_element_id(stream):
    """Read the Element ID

    :param stream: file-like object from which to read
    :raise ReadError: when not all the required bytes could be read
    :return: the id of the element
    :rtype: int

    """
    char = _read(stream, 1)
    byte = ord(char)
    if byte & 0x80:
        return byte
    elif byte & 0x40:
        return unpack('>H', char + _read(stream, 1))[0]
    elif byte & 0x20:
        b, h = unpack('>BH', char + _read(stream, 2))
        return b * 2 ** 16 + h
    elif byte & 0x10:
        return unpack('>L', char + _read(stream, 3))[0]
    else:
        ValueError('Not an Element ID')


def read_element_size(stream):
    """Read the Element Size

    :param stream: file-like object from which to read
    :raise ReadError: when not all the required bytes could be read
    :return: the size of element's data
    :rtype: int

    """
    char = _read(stream, 1)
    byte = ord(char)
    if byte & 0x80:
        return unpack('>B', bytes((byte ^ 0x80,)))[0]
    elif byte & 0x40:
        return unpack('>H', bytes((byte ^ 0x40,)) + _read(stream, 1))[0]
    elif byte & 0x20:
        b, h = unpack('>BH', bytes((byte ^ 0x20,)) + _read(stream, 2))
        return b * 2 ** 16 + h
    elif byte & 0x10:
        return unpack('>L', bytes((byte ^ 0x10,)) + _read(stream, 3))[0]
    elif byte & 0x08:
        b, l = unpack('>BL', bytes((byte ^ 0x08,)) + _read(stream, 4))
        return b * 2 ** 32 + l
    elif byte & 0x04:
        h, l = unpack('>HL', bytes((byte ^ 0x04,)) + _read(stream, 5))
        return h * 2 ** 32 + l
    elif byte & 0x02:
        b, h, l = unpack('>BHL', bytes((byte ^ 0x02,)) + _read(stream, 6))
        return b * 2 ** 48 + h * 2 ** 32 + l
    elif byte & 0x01:
        return unpack('>Q', bytes((byte ^ 0x01,)) + _read(stream, 7))[0]
    else:
        ValueError('Not an Element Size')


def read_element_integer(stream, size):
    """Read the Element Data of type :data:`INTEGER`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: the read integer
    :rtype: int

    """
    if size == 1:
        return unpack('>b', _read(stream, 1))[0]
    elif size == 2:
        return unpack('>h', _read(stream, 2))[0]
    elif size == 3:
        b, h = unpack('>bH', _read(stream, 3))
        return b * 2 ** 16 + h
    elif size == 4:
        return unpack('>l', _read(stream, 4))[0]
    elif size == 5:
        b, l = unpack('>bL', _read(stream, 5))
        return b * 2 ** 32 + l
    elif size == 6:
        h, l = unpack('>hL', _read(stream, 6))
        return h * 2 ** 32 + l
    elif size == 7:
        b, h, l = unpack('>bHL', _read(stream, 7))
        return b * 2 ** 48 + h * 2 ** 32 + l
    elif size == 8:
        return unpack('>q', _read(stream, 8))[0]
    else:
        raise SizeError(size)


def read_element_uinteger(stream, size):
    """Read the Element Data of type :data:`UINTEGER`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: the read unsigned integer
    :rtype: int

    """
    if size == 1:
        return unpack('>B', _read(stream, 1))[0]
    elif size == 2:
        return unpack('>H', _read(stream, 2))[0]
    elif size == 3:
        b, h = unpack('>BH', _read(stream, 3))
        return b * 2 ** 16 + h
    elif size == 4:
        return unpack('>L', _read(stream, 4))[0]
    elif size == 5:
        b, l = unpack('>BL', _read(stream, 5))
        return b * 2 ** 32 + l
    elif size == 6:
        h, l = unpack('>HL', _read(stream, 6))
        return h * 2 ** 32 + l
    elif size == 7:
        b, h, l = unpack('>BHL', _read(stream, 7))
        return b * 2 ** 48 + h * 2 ** 32 + l
    elif size == 8:
        return unpack('>Q', _read(stream, 8))[0]
    else:
        raise SizeError(size)


def read_element_float(stream, size):
    """Read the Element Data of type :data:`FLOAT`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: the read float
    :rtype: float

    """
    if size == 4:
        return unpack('>f', _read(stream, 4))[0]
    elif size == 8:
        return unpack('>d', _read(stream, 8))[0]
    else:
        raise SizeError(size)


def read_element_string(stream, size):
    """Read the Element Data of type :data:`STRING`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: the read ascii-decoded string
    :rtype: unicode

    """
    return _read(stream, size).decode('ascii')


def read_element_unicode(stream, size):
    """Read the Element Data of type :data:`UNICODE`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: the read utf-8-decoded string
    :rtype: unicode

    """
    return _read(stream, size).decode('utf-8')


def read_element_date(stream, size):
    """Read the Element Data of type :data:`DATE`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: the read date
    :rtype: datetime

    """
    if size != 8:
        raise SizeError(size)
    nanoseconds = unpack('>q', _read(stream, 8))[0]
    return datetime(2001, 1, 1, 0, 0, 0, 0, None) + timedelta(microseconds=nanoseconds // 1000)


def read_element_binary(stream, size):
    """Read the Element Data of type :data:`BINARY`

    :param stream: file-like object from which to read
    :param int size: size of element's data
    :raise ReadError: when not all the required bytes could be read
    :raise SizeError: if size is incorrect
    :return: raw binary data
    :rtype: bytes

    """
    return BytesIO(stream.read(size))



# -*- coding: utf-8 -*-
from .exceptions import ParserError, MalformedMKVError
from .parsers import ebml
from datetime import timedelta
import logging


__all__ = ['VIDEO_TRACK', 'AUDIO_TRACK', 'SUBTITLE_TRACK', 'MKV', 'Info', 'Track', 'VideoTrack',
           'AudioTrack', 'SubtitleTrack', 'Tag', 'SimpleTag', 'Chapter']
logger = logging.getLogger(__name__)


# Track types
VIDEO_TRACK, AUDIO_TRACK, SUBTITLE_TRACK = 0x01, 0x02, 0x11


class MKV(object):
    """Matroska Video file

    :param stream: seekable file-like object

    """
    def __init__(self, stream):
        # default attributes
        self.info = None
        self.video_tracks = []
        self.audio_tracks = []
        self.subtitle_tracks = []
        self.chapters = []
        self.tags = []

        try:
            # get the Segment element
            logger.info('Reading Segment element')
            specs = ebml.get_matroska_specs()
            segments = ebml.parse(stream, specs, ignore_element_names=['EBML'], max_level=0)
            if not segments:
                raise MalformedMKVError('No Segment found')
            if len(segments) > 1:
                logger.warning('%d segments found, using the first one', len(segments))
            segment = segments[0]

            # get the SeekHead element
            logger.info('Reading SeekHead element')
            stream.seek(segment.position)
            seek_head = ebml.parse_element(stream, specs)
            if seek_head.name != 'SeekHead':
                raise MalformedMKVError('No SeekHead found')
            seek_head.load(stream, specs, ignore_element_names=['Void', 'CRC-32'])

            # parse interesting informations
            for seek in seek_head:
                element_id = ebml.read_element_id(seek['SeekID'].data)
                element_name = specs[element_id][1]
                element_position = seek['SeekPosition'].data + segment.position
                if element_name == 'Info':
                    logger.info('Processing element %s from SeekHead at position %d', element_name, element_position)
                    stream.seek(element_position)
                    self.info = Info.fromelement(ebml.parse_element(stream, specs, True, ignore_element_names=['Void', 'CRC-32']))
                elif element_name == 'Tracks':
                    logger.info('Processing element %s from SeekHead at position %d', element_name, element_position)
                    stream.seek(element_position)
                    tracks = ebml.parse_element(stream, specs, True, ignore_element_names=['Void', 'CRC-32'])
                    self.video_tracks = [VideoTrack.fromelement(t) for t in tracks if t['TrackType'].data == VIDEO_TRACK]
                    self.audio_tracks = [AudioTrack.fromelement(t) for t in tracks if t['TrackType'].data == AUDIO_TRACK]
                    self.subtitle_tracks = [SubtitleTrack.fromelement(t) for t in tracks if t['TrackType'].data == SUBTITLE_TRACK]
                elif element_name == 'Chapters':
                    logger.info('Processing element %s from SeekHead at position %d', element_name, element_position)
                    stream.seek(element_position)
                    self.chapters = [Chapter.fromelement(c) for c in ebml.parse_element(stream, specs, True, ignore_element_names=['Void', 'CRC-32'])[0] if c.name == 'ChapterAtom']
                elif element_name == 'Tags':
                    logger.info('Processing element %s from SeekHead at position %d', element_name, element_position)
                    stream.seek(element_position)
                    self.tags = [Tag.fromelement(t) for t in ebml.parse_element(stream, specs, True, ignore_element_names=['Void', 'CRC-32'])]
                else:
                    logger.debug('Element %s ignored', element_name)
                    continue
        except ParserError as e:
            raise MalformedMKVError('Parsing error: %s' % e)

    def to_dict(self):
        return {'info': self.info.__dict__, 'video_tracks': [t.__dict__ for t in self.video_tracks],
                'audio_tracks': [t.__dict__ for t in self.audio_tracks], 'subtitle_tracks': [t.__dict__ for t in self.subtitle_tracks],
                'chapters': [c.__dict__ for c in self.chapters], 'tags': [t.__dict__ for t in self.tags]}

    def __repr__(self):
        return '<%s [%r, %r, %r, %r]>' % (self.__class__.__name__, self.info, self.video_tracks, self.audio_tracks, self.subtitle_tracks)


class Info(object):
    """Object for the Info EBML element"""
    def __init__(self, title=None, duration=None, date_utc=None, timecode_scale=None, muxing_app=None, writing_app=None):
        self.title = title
        self.duration = timedelta(microseconds=duration * (timecode_scale or 1000000) // 1000) if duration else None
        self.date_utc = date_utc
        self.muxing_app = muxing_app
        self.writing_app = writing_app

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`Info` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the Info element
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        title = element.get('Title')
        duration = element.get('Duration')
        date_utc = element.get('DateUTC')
        timecode_scale = element.get('TimecodeScale')
        muxing_app = element.get('MuxingApp')
        writing_app = element.get('WritingApp')
        return cls(title, duration, date_utc, timecode_scale, muxing_app, writing_app)

    def __repr__(self):
        return '<%s [title=%r, duration=%s, date=%s]>' % (self.__class__.__name__, self.title, self.duration, self.date_utc)

    def __str__(self):
        return repr(self.__dict__)


class Track(object):
    """Base object for the Tracks EBML element"""
    def __init__(self, type=None, number=None, name=None, language=None, enabled=None, default=None, forced=None, lacing=None,
                 codec_id=None, codec_name=None):
        self.type = type
        self.number = number
        self.name = name
        self.language = language
        self.enabled = enabled
        self.default = default
        self.forced = forced
        self.lacing = lacing
        self.codec_id = codec_id
        self.codec_name = codec_name

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`Track` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the Track element
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        type = element.get('TrackType')
        number = element.get('TrackNumber', 0)
        name = element.get('Name')
        language = element.get('Language')
        enabled = bool(element.get('FlagEnabled', 1))
        default = bool(element.get('FlagDefault', 1))
        forced = bool(element.get('FlagForced', 0))
        lacing = bool(element.get('FlagLacing', 1))
        codec_id = element.get('CodecID')
        codec_name = element.get('CodecName')
        return cls(type=type, number=number, name=name, language=language, enabled=enabled, default=default,
                   forced=forced, lacing=lacing, codec_id=codec_id, codec_name=codec_name)

    def __repr__(self):
        return '<%s [%d, name=%r, language=%s]>' % (self.__class__.__name__, self.number, self.name, self.language)

    def __str__(self):
        return str(self.__dict__)


class VideoTrack(Track):
    """Object for the Tracks EBML element with :data:`VIDEO_TRACK` TrackType"""
    def __init__(self, width=0, height=0, interlaced=False, stereo_mode=None, crop=None,
                 display_width=None, display_height=None, display_unit=None, aspect_ratio_type=None, **kwargs):
        super(VideoTrack, self).__init__(**kwargs)
        self.width = width
        self.height = height
        self.interlaced = interlaced
        self.stereo_mode = stereo_mode
        self.crop = crop
        self.display_width = display_width
        self.display_height = display_height
        self.display_unit = display_unit
        self.aspect_ratio_type = aspect_ratio_type

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`VideoTrack` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the Track element with :data:`VIDEO_TRACK` TrackType
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        videotrack = super(VideoTrack, cls).fromelement(element)
        videotrack.width = element['Video'].get('PixelWidth', 0)
        videotrack.height = element['Video'].get('PixelHeight', 0)
        videotrack.interlaced = bool(element['Video'].get('FlagInterlaced', False))
        videotrack.stereo_mode = element['Video'].get('StereoMode')
        videotrack.crop = {}
        if 'PixelCropTop' in element['Video']:
            videotrack.crop['top'] = element['Video']['PixelCropTop']
        if 'PixelCropBottom' in element['Video']:
            videotrack.crop['bottom'] = element['Video']['PixelCropBottom']
        if 'PixelCropLeft' in element['Video']:
            videotrack.crop['left'] = element['Video']['PixelCropLeft']
        if 'PixelCropRight' in element['Video']:
            videotrack.crop['right'] = element['Video']['PixelCropRight']
        videotrack.display_width = element['Video'].get('DisplayWidth')
        videotrack.display_height = element['Video'].get('DisplayHeight')
        videotrack.display_unit = element['Video'].get('DisplayUnit')
        videotrack.aspect_ratio_type = element['Video'].get('AspectRatioType')
        return videotrack

    def __repr__(self):
        return '<%s [%d, %dx%d, %s, name=%r, language=%s]>' % (self.__class__.__name__, self.number, self.width, self.height,
                                                            self.codec_id, self.name, self.language)

    def __str__(self):
        return str(self.__dict__)


class AudioTrack(Track):
    """Object for the Tracks EBML element with :data:`AUDIO_TRACK` TrackType"""
    def __init__(self, sampling_frequency=None, channels=None, output_sampling_frequency=None, bit_depth=None, **kwargs):
        super(AudioTrack, self).__init__(**kwargs)
        self.sampling_frequency = sampling_frequency
        self.channels = channels
        self.output_sampling_frequency = output_sampling_frequency
        self.bit_depth = bit_depth

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`AudioTrack` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the Track element with :data:`AUDIO_TRACK` TrackType
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        audiotrack = super(AudioTrack, cls).fromelement(element)
        audiotrack.sampling_frequency = element['Audio'].get('SamplingFrequency', 8000.0)
        audiotrack.channels = element['Audio'].get('Channels', 1)
        audiotrack.output_sampling_frequency = element['Audio'].get('OutputSamplingFrequency')
        audiotrack.bit_depth = element['Audio'].get('BitDepth')
        return audiotrack

    def __repr__(self):
        return '<%s [%d, %d channel(s), %.0fHz, %s, name=%r, language=%s]>' % (self.__class__.__name__, self.number, self.channels,
                                                                            self.sampling_frequency, self.codec_id, self.name, self.language)


class SubtitleTrack(Track):
    """Object for the Tracks EBML element with :data:`SUBTITLE_TRACK` TrackType"""
    pass


class Tag(object):
    """Object for the Tag EBML element"""
    def __init__(self, targets=None, simpletags=None):
        self.targets = targets if targets is not None else []
        self.simpletags = simpletags if simpletags is not None else []

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`Tag` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the Tag element
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        targets = element['Targets'] if 'Targets' in element else []
        simpletags = [SimpleTag.fromelement(s) for s in element if s.name == 'SimpleTag']
        return cls(targets, simpletags)

    def __repr__(self):
        return '<%s [targets=%r, simpletags=%r]>' % (self.__class__.__name__, self.targets, self.simpletags)


class SimpleTag(object):
    """Object for the SimpleTag EBML element"""
    def __init__(self, name, language='und', default=True, string=None, binary=None):
        self.name = name
        self.language = language
        self.default = default
        self.string = string
        self.binary = binary

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`SimpleTag` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the SimpleTag element
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        name = element.get('TagName')
        language = element.get('TagLanguage', 'und')
        default = element.get('TagDefault', True)
        string = element.get('TagString')
        binary = element.get('TagBinary')
        return cls(name, language, default, string, binary)

    def __repr__(self):
        return '<%s [%s, language=%s, default=%s, string=%s]>' % (self.__class__.__name__, self.name, self.language, self.default, self.string)


class Chapter(object):
    """Object for the ChapterAtom and ChapterDisplay EBML element

    .. note::
        For the sake of simplicity, it is assumed that the ChapterAtom element
        has no more than 1 ChapterDisplay child element and informations it contains
        are merged into the :class:`Chapter`

    """
    def __init__(self, start, hidden=False, enabled=False, end=None, string=None, language=None):
        self.start = start
        self.hidden = hidden
        self.enabled = enabled
        self.end = end
        self.string = string
        self.language = language

    @classmethod
    def fromelement(cls, element):
        """Load the :class:`Chapter` from an :class:`~enzyme.parsers.ebml.Element`

        :param element: the ChapterAtom element
        :type element: :class:`~enzyme.parsers.ebml.Element`

        """
        start = timedelta(microseconds=element.get('ChapterTimeStart') // 1000)
        hidden = element.get('ChapterFlagHidden', False)
        enabled = element.get('ChapterFlagEnabled', True)
        end = element.get('ChapterTimeEnd')
        chapterdisplays = [c for c in element if c.name == 'ChapterDisplay']
        if len(chapterdisplays) > 1:
            logger.warning('More than 1 (%d) ChapterDisplay element in the ChapterAtom, using the first one', len(chapterdisplays))
        if chapterdisplays:
            string = chapterdisplays[0].get('ChapString')
            language = chapterdisplays[0].get('ChapLanguage')
        return cls(start, hidden, enabled, end, string, language)

    def __repr__(self):
        return '<%s [%s, enabled=%s]>' % (self.__class__.__name__, self.start, self.enabled)



# -*- coding: utf-8 -*-
from datetime import timedelta, datetime
from enzyme.mkv import MKV, VIDEO_TRACK, AUDIO_TRACK, SUBTITLE_TRACK
import io
import os.path
import requests
import unittest
import zipfile


# Test directory
TEST_DIR = os.path.join(os.path.dirname(__file__), os.path.splitext(__file__)[0])


def setUpModule():
    if not os.path.exists(TEST_DIR):
        r = requests.get('http://downloads.sourceforge.net/project/matroska/test_files/matroska_test_w1_1.zip')
        with zipfile.ZipFile(io.BytesIO(r.content), 'r') as f:
            f.extractall(TEST_DIR)


class MKVTestCase(unittest.TestCase):
    def test_test1(self):
        stream = io.open(os.path.join(TEST_DIR, 'test1.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(minutes=1, seconds=27, milliseconds=336))
        self.assertTrue(mkv.info.date_utc == datetime(2010, 8, 21, 7, 23, 3))
        self.assertTrue(mkv.info.muxing_app == 'libebml2 v0.10.0 + libmatroska2 v0.10.1')
        self.assertTrue(mkv.info.writing_app == 'mkclean 0.5.5 ru from libebml v1.0.0 + libmatroska v1.0.0 + mkvmerge v4.1.1 (\'Bouncin\' Back\') built on Jul  3 2010 22:54:08')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == True)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MS/VFW/FOURCC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 854)
        self.assertTrue(mkv.video_tracks[0].height == 480)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width is None)
        self.assertTrue(mkv.video_tracks[0].display_height is None)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio track
        self.assertTrue(len(mkv.audio_tracks) == 1)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language == 'und')
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == True)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_MPEG/L3')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 0)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Big Buck Bunny - test 1')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File1, basic MPEG4.2 and MP3 with only SimpleBlock')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)

    def test_test2(self):
        stream = io.open(os.path.join(TEST_DIR, 'test2.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(seconds=47, milliseconds=509))
        self.assertTrue(mkv.info.date_utc == datetime(2011, 6, 2, 12, 45, 20))
        self.assertTrue(mkv.info.muxing_app == 'libebml2 v0.21.0 + libmatroska2 v0.22.1')
        self.assertTrue(mkv.info.writing_app == 'mkclean 0.8.3 ru from libebml2 v0.10.0 + libmatroska2 v0.10.1 + mkclean 0.5.5 ru from libebml v1.0.0 + libmatroska v1.0.0 + mkvmerge v4.1.1 (\'Bouncin\' Back\') built on Jul  3 2010 22:54:08')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == True)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MPEG4/ISO/AVC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 1024)
        self.assertTrue(mkv.video_tracks[0].height == 576)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width == 1354)
        self.assertTrue(mkv.video_tracks[0].display_height is None)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio track
        self.assertTrue(len(mkv.audio_tracks) == 1)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language == 'und')
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == True)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_AAC')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 0)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Elephant Dream - test 2')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File 2, 100,000 timecode scale, odd aspect ratio, and CRC-32. Codecs are AVC and AAC')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)

    def test_test3(self):
        stream = io.open(os.path.join(TEST_DIR, 'test3.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(seconds=49, milliseconds=64))
        self.assertTrue(mkv.info.date_utc == datetime(2010, 8, 21, 21, 43, 25))
        self.assertTrue(mkv.info.muxing_app == 'libebml2 v0.11.0 + libmatroska2 v0.10.1')
        self.assertTrue(mkv.info.writing_app == 'mkclean 0.5.5 ro from libebml v1.0.0 + libmatroska v1.0.0 + mkvmerge v4.1.1 (\'Bouncin\' Back\') built on Jul  3 2010 22:54:08')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == True)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MPEG4/ISO/AVC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 1024)
        self.assertTrue(mkv.video_tracks[0].height == 576)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width is None)
        self.assertTrue(mkv.video_tracks[0].display_height is None)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio track
        self.assertTrue(len(mkv.audio_tracks) == 1)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language is None)
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == True)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_MPEG/L3')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 0)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Elephant Dream - test 3')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File 3, header stripping on the video track and no SimpleBlock')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)

    def test_test5(self):
        stream = io.open(os.path.join(TEST_DIR, 'test5.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(seconds=46, milliseconds=665))
        self.assertTrue(mkv.info.date_utc == datetime(2010, 8, 21, 18, 6, 43))
        self.assertTrue(mkv.info.muxing_app == 'libebml v1.0.0 + libmatroska v1.0.0')
        self.assertTrue(mkv.info.writing_app == 'mkvmerge v4.0.0 (\'The Stars were mine\') built on Jun  6 2010 16:18:42')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == True)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MPEG4/ISO/AVC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 1024)
        self.assertTrue(mkv.video_tracks[0].height == 576)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width == 1024)
        self.assertTrue(mkv.video_tracks[0].display_height == 576)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio tracks
        self.assertTrue(len(mkv.audio_tracks) == 2)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language == 'und')
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == True)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_AAC')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        self.assertTrue(mkv.audio_tracks[1].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[1].number == 10)
        self.assertTrue(mkv.audio_tracks[1].name == 'Commentary')
        self.assertTrue(mkv.audio_tracks[1].language is None)
        self.assertTrue(mkv.audio_tracks[1].enabled == True)
        self.assertTrue(mkv.audio_tracks[1].default == False)
        self.assertTrue(mkv.audio_tracks[1].forced == False)
        self.assertTrue(mkv.audio_tracks[1].lacing == True)
        self.assertTrue(mkv.audio_tracks[1].codec_id == 'A_AAC')
        self.assertTrue(mkv.audio_tracks[1].codec_name is None)
        self.assertTrue(mkv.audio_tracks[1].sampling_frequency == 22050.0)
        self.assertTrue(mkv.audio_tracks[1].channels == 1)
        self.assertTrue(mkv.audio_tracks[1].output_sampling_frequency == 44100.0)
        self.assertTrue(mkv.audio_tracks[1].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 8)
        self.assertTrue(mkv.subtitle_tracks[0].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[0].number == 3)
        self.assertTrue(mkv.subtitle_tracks[0].name is None)
        self.assertTrue(mkv.subtitle_tracks[0].language is None)
        self.assertTrue(mkv.subtitle_tracks[0].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[0].default == True)
        self.assertTrue(mkv.subtitle_tracks[0].forced == False)
        self.assertTrue(mkv.subtitle_tracks[0].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[0].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[0].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[1].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[1].number == 4)
        self.assertTrue(mkv.subtitle_tracks[1].name is None)
        self.assertTrue(mkv.subtitle_tracks[1].language == 'hun')
        self.assertTrue(mkv.subtitle_tracks[1].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[1].default == False)
        self.assertTrue(mkv.subtitle_tracks[1].forced == False)
        self.assertTrue(mkv.subtitle_tracks[1].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[1].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[1].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[2].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[2].number == 5)
        self.assertTrue(mkv.subtitle_tracks[2].name is None)
        self.assertTrue(mkv.subtitle_tracks[2].language == 'ger')
        self.assertTrue(mkv.subtitle_tracks[2].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[2].default == False)
        self.assertTrue(mkv.subtitle_tracks[2].forced == False)
        self.assertTrue(mkv.subtitle_tracks[2].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[2].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[2].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[3].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[3].number == 6)
        self.assertTrue(mkv.subtitle_tracks[3].name is None)
        self.assertTrue(mkv.subtitle_tracks[3].language == 'fre')
        self.assertTrue(mkv.subtitle_tracks[3].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[3].default == False)
        self.assertTrue(mkv.subtitle_tracks[3].forced == False)
        self.assertTrue(mkv.subtitle_tracks[3].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[3].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[3].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[4].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[4].number == 8)
        self.assertTrue(mkv.subtitle_tracks[4].name is None)
        self.assertTrue(mkv.subtitle_tracks[4].language == 'spa')
        self.assertTrue(mkv.subtitle_tracks[4].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[4].default == False)
        self.assertTrue(mkv.subtitle_tracks[4].forced == False)
        self.assertTrue(mkv.subtitle_tracks[4].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[4].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[4].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[5].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[5].number == 9)
        self.assertTrue(mkv.subtitle_tracks[5].name is None)
        self.assertTrue(mkv.subtitle_tracks[5].language == 'ita')
        self.assertTrue(mkv.subtitle_tracks[5].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[5].default == False)
        self.assertTrue(mkv.subtitle_tracks[5].forced == False)
        self.assertTrue(mkv.subtitle_tracks[5].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[5].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[5].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[6].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[6].number == 11)
        self.assertTrue(mkv.subtitle_tracks[6].name is None)
        self.assertTrue(mkv.subtitle_tracks[6].language == 'jpn')
        self.assertTrue(mkv.subtitle_tracks[6].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[6].default == False)
        self.assertTrue(mkv.subtitle_tracks[6].forced == False)
        self.assertTrue(mkv.subtitle_tracks[6].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[6].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[6].codec_name is None)
        self.assertTrue(mkv.subtitle_tracks[7].type == SUBTITLE_TRACK)
        self.assertTrue(mkv.subtitle_tracks[7].number == 7)
        self.assertTrue(mkv.subtitle_tracks[7].name is None)
        self.assertTrue(mkv.subtitle_tracks[7].language == 'und')
        self.assertTrue(mkv.subtitle_tracks[7].enabled == True)
        self.assertTrue(mkv.subtitle_tracks[7].default == False)
        self.assertTrue(mkv.subtitle_tracks[7].forced == False)
        self.assertTrue(mkv.subtitle_tracks[7].lacing == False)
        self.assertTrue(mkv.subtitle_tracks[7].codec_id == 'S_TEXT/UTF8')
        self.assertTrue(mkv.subtitle_tracks[7].codec_name is None)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Big Buck Bunny - test 8')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File 8, secondary audio commentary track, misc subtitle tracks')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)

    def test_test6(self):
        stream = io.open(os.path.join(TEST_DIR, 'test6.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(seconds=87, milliseconds=336))
        self.assertTrue(mkv.info.date_utc == datetime(2010, 8, 21, 16, 31, 55))
        self.assertTrue(mkv.info.muxing_app == 'libebml2 v0.10.1 + libmatroska2 v0.10.1')
        self.assertTrue(mkv.info.writing_app == 'mkclean 0.5.5 r from libebml v1.0.0 + libmatroska v1.0.0 + mkvmerge v4.0.0 (\'The Stars were mine\') built on Jun  6 2010 16:18:42')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == False)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MS/VFW/FOURCC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 854)
        self.assertTrue(mkv.video_tracks[0].height == 480)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width is None)
        self.assertTrue(mkv.video_tracks[0].display_height is None)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio track
        self.assertTrue(len(mkv.audio_tracks) == 1)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language == 'und')
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == False)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_MPEG/L3')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 0)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Big Buck Bunny - test 6')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File 6, random length to code the size of Clusters and Blocks, no Cues for seeking')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)

    def test_test7(self):
        stream = io.open(os.path.join(TEST_DIR, 'test7.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(seconds=37, milliseconds=43))
        self.assertTrue(mkv.info.date_utc == datetime(2010, 8, 21, 17, 0, 23))
        self.assertTrue(mkv.info.muxing_app == 'libebml2 v0.10.1 + libmatroska2 v0.10.1')
        self.assertTrue(mkv.info.writing_app == 'mkclean 0.5.5 r from libebml v1.0.0 + libmatroska v1.0.0 + mkvmerge v4.0.0 (\'The Stars were mine\') built on Jun  6 2010 16:18:42')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == False)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MPEG4/ISO/AVC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 1024)
        self.assertTrue(mkv.video_tracks[0].height == 576)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width is None)
        self.assertTrue(mkv.video_tracks[0].display_height is None)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio track
        self.assertTrue(len(mkv.audio_tracks) == 1)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language == 'und')
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == False)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_AAC')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 0)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Big Buck Bunny - test 7')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File 7, junk elements are present at the beggining or end of clusters, the parser should skip it. There is also a damaged element at 451418')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)

    def test_test8(self):
        stream = io.open(os.path.join(TEST_DIR, 'test8.mkv'), 'rb')
        mkv = MKV(stream)
        # info
        self.assertTrue(mkv.info.title is None)
        self.assertTrue(mkv.info.duration == timedelta(seconds=47, milliseconds=341))
        self.assertTrue(mkv.info.date_utc == datetime(2010, 8, 21, 17, 22, 14))
        self.assertTrue(mkv.info.muxing_app == 'libebml2 v0.10.1 + libmatroska2 v0.10.1')
        self.assertTrue(mkv.info.writing_app == 'mkclean 0.5.5 r from libebml v1.0.0 + libmatroska v1.0.0 + mkvmerge v4.0.0 (\'The Stars were mine\') built on Jun  6 2010 16:18:42')
        # video track
        self.assertTrue(len(mkv.video_tracks) == 1)
        self.assertTrue(mkv.video_tracks[0].type == VIDEO_TRACK)
        self.assertTrue(mkv.video_tracks[0].number == 1)
        self.assertTrue(mkv.video_tracks[0].name is None)
        self.assertTrue(mkv.video_tracks[0].language == 'und')
        self.assertTrue(mkv.video_tracks[0].enabled == True)
        self.assertTrue(mkv.video_tracks[0].default == False)
        self.assertTrue(mkv.video_tracks[0].forced == False)
        self.assertTrue(mkv.video_tracks[0].lacing == False)
        self.assertTrue(mkv.video_tracks[0].codec_id == 'V_MPEG4/ISO/AVC')
        self.assertTrue(mkv.video_tracks[0].codec_name is None)
        self.assertTrue(mkv.video_tracks[0].width == 1024)
        self.assertTrue(mkv.video_tracks[0].height == 576)
        self.assertTrue(mkv.video_tracks[0].interlaced == False)
        self.assertTrue(mkv.video_tracks[0].stereo_mode is None)
        self.assertTrue(mkv.video_tracks[0].crop == {})
        self.assertTrue(mkv.video_tracks[0].display_width is None)
        self.assertTrue(mkv.video_tracks[0].display_height is None)
        self.assertTrue(mkv.video_tracks[0].display_unit is None)
        self.assertTrue(mkv.video_tracks[0].aspect_ratio_type is None)
        # audio track
        self.assertTrue(len(mkv.audio_tracks) == 1)
        self.assertTrue(mkv.audio_tracks[0].type == AUDIO_TRACK)
        self.assertTrue(mkv.audio_tracks[0].number == 2)
        self.assertTrue(mkv.audio_tracks[0].name is None)
        self.assertTrue(mkv.audio_tracks[0].language == 'und')
        self.assertTrue(mkv.audio_tracks[0].enabled == True)
        self.assertTrue(mkv.audio_tracks[0].default == False)
        self.assertTrue(mkv.audio_tracks[0].forced == False)
        self.assertTrue(mkv.audio_tracks[0].lacing == True)
        self.assertTrue(mkv.audio_tracks[0].codec_id == 'A_AAC')
        self.assertTrue(mkv.audio_tracks[0].codec_name is None)
        self.assertTrue(mkv.audio_tracks[0].sampling_frequency == 48000.0)
        self.assertTrue(mkv.audio_tracks[0].channels == 2)
        self.assertTrue(mkv.audio_tracks[0].output_sampling_frequency is None)
        self.assertTrue(mkv.audio_tracks[0].bit_depth is None)
        # subtitle track
        self.assertTrue(len(mkv.subtitle_tracks) == 0)
        # chapters
        self.assertTrue(len(mkv.chapters) == 0)
        # tags
        self.assertTrue(len(mkv.tags) == 1)
        self.assertTrue(len(mkv.tags[0].simpletags) == 3)
        self.assertTrue(mkv.tags[0].simpletags[0].name == 'TITLE')
        self.assertTrue(mkv.tags[0].simpletags[0].default == True)
        self.assertTrue(mkv.tags[0].simpletags[0].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[0].string == 'Big Buck Bunny - test 8')
        self.assertTrue(mkv.tags[0].simpletags[0].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[1].name == 'DATE_RELEASED')
        self.assertTrue(mkv.tags[0].simpletags[1].default == True)
        self.assertTrue(mkv.tags[0].simpletags[1].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[1].string == '2010')
        self.assertTrue(mkv.tags[0].simpletags[1].binary is None)
        self.assertTrue(mkv.tags[0].simpletags[2].name == 'COMMENT')
        self.assertTrue(mkv.tags[0].simpletags[2].default == True)
        self.assertTrue(mkv.tags[0].simpletags[2].language == 'und')
        self.assertTrue(mkv.tags[0].simpletags[2].string == 'Matroska Validation File 8, audio missing between timecodes 6.019s and 6.360s')
        self.assertTrue(mkv.tags[0].simpletags[2].binary is None)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(MKVTestCase))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages


setup(name='enzyme',
    version='0.4.0',
    license='Apache 2.0',
    description='Python video metadata parser',
    long_description=open('README.rst').read() + '\n\n' + open('HISTORY.rst').read(),
    keywords='parser video metadata mkv',
    url='https://github.com/Diaoul/enzyme',
    author='Antoine Bertin',
    author_email='diaoulael@gmail.com',
    packages=find_packages(),
    package_data={'enzyme.parsers.ebml': ['specs/matroska.xml'], 'enzyme.tests': ['parsers/ebml/test1.mkv.yml']},
    classifiers=['Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Multimedia :: Video',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    test_suite='enzyme.tests.suite',
    tests_require=['requests', 'PyYAML'])
